#! /bin/sh
##
# Renew Splunk's self-signed server SSL certificates with a new expiration date
# but old private key.
##

set -e # stop on error
#set -x # uncomment for debugging

########## Default settings ###################################################

DAYS=3650
PASSWORD="password"

CA_COUNTRY="US"
CA_STATE="CA"
CA_CITY="San Francisco"
CA_ORGANIZATION="Splunk"
CA_COMMON_NAME="SplunkCommonCA"
CA_E_MAIL="support@splunk.com"

COMMON_NAME="SplunkServerDefaultCert"

SPLUNK_CERT_EXPIRATION_YEAR=2016

########## Functions ##########################################################

my_basename() {
  ##
  # Autoconf, 11.15:
  #
  # basename
  #   Not all hosts have a working basename. You can use expr instead.
  ##
  expr "//$1" : '.*/\(.*\)'
}

info() {
  echo "$ME: $1"
}

error() {
  LOCAL_MSG="$1"
  LOCAL_STATUS="$2"
  [ "x$LOCAL_STATUS" = "x" ] && LOCAL_STATUS=99
  echo "$ME: error: $LOCAL_MSG" >&2
  exit $LOCAL_STATUS
}

assert_exists() {
  [ -e "$1" ] || error "$1 not found" 10
}

extract_x509_first_cert() {
  LOCAL_FROM="$1"
  LOCAL_TO="$2"
  assert_exists "$LOCAL_FROM"
  sed -n '1,/END CERTIFICATE/p' "$LOCAL_FROM" > "$LOCAL_TO"
  grep CERTIFICATE "$LOCAL_TO" >/dev/null 2>/dev/null ||
    error "$LOCAL_FROM doesn't contain a certificate" 11
}

extract_x509_private_key() {
  LOCAL_FROM="$1"
  LOCAL_TO="$2"
  assert_exists "$LOCAL_FROM"
  # We don't use "openssl pkey" since that's not available in older openssl.
  sed -n '/BEGIN.*PRIVATE/,/END.*PRIVATE/p' "$LOCAL_FROM" > "$LOCAL_TO"
  grep PRIVATE "$LOCAL_TO" >/dev/null 2>/dev/null ||
    error "$LOCAL_FROM doesn't contain a private key" 12
}

get_x509_organization() {
  assert_exists "$1"
  openssl x509 -in "$1" -text -noout | sed -n '/Issuer/s/.*O=\([^,]*\).*/\1/p'
}

get_x509_not_after_year() {
  assert_exists "$1"
  openssl x509 -in "$1" -text -noout |
    sed -n '/Not After/s/.* \(20[0-9][0-9]\) .*/\1/p'
}

assert_is_Splunk_cert() {
  LOCAL_FILE="$1"
  LOCAL_ORG=`get_x509_organization "$LOCAL_FILE"`
  [ "x$LOCAL_ORG" = "x$CA_ORGANIZATION" ] ||
    error "$LOCAL_FILE is not a Splunk-issued certificate" 13
  get_x509_not_after_year "$LOCAL_FILE"
}

assert_Splunk_cert_is_new() {
  LOCAL_FILE="$1"
  LOCAL_EXP_YEAR=`assert_is_Splunk_cert "$LOCAL_FILE"`
  [ "$LOCAL_EXP_YEAR" -gt "$SPLUNK_CERT_EXPIRATION_YEAR" ] ||
    error "$LOCAL_FILE is not a renewed Splunk certificate" 14
}

assert_Splunk_cert_is_old() {
  LOCAL_FILE="$1"
  LOCAL_EXP_YEAR=`assert_is_Splunk_cert "$LOCAL_FILE"`
  [ "$LOCAL_EXP_YEAR" -le "$SPLUNK_CERT_EXPIRATION_YEAR" ] ||
    error "$LOCAL_FILE is already a renewed Splunk certificate" 15
}

backup() {
  LOCAL_FILE="$1"
  LOCAL_ORIG="$LOCAL_FILE.orig"
  [ -e "$LOCAL_FILE" -a ! -e "$LOCAL_ORIG" ] && cp "$LOCAL_FILE" "$LOCAL_ORIG"
  return 0
}

restore() {
  LOCAL_FILE="$1"
  LOCAL_ORIG="$LOCAL_FILE.orig"
  cp "$LOCAL_ORIG" "$LOCAL_FILE"
}

usage() {
  cat >&2 <<END
usage: $ME [options]

Renew Splunk's self-signed SSL certificates.

optional arguments:
  -help                show this help message and exit
  -defaultCA           renew default CA certificates [default: false]
  -liveCA              renew live CA certificates [default: false]
  -opensslConf <path>  set/override value of \$OPENSSL_CONF
  -serverCert          renew server.pem certificate [default: false]
  -splunkHome <dir>    set/override value of \$SPLUNK_HOME

At least one of -defaultCA, -liveCA, or -serverCert is required;
order should be -defaultCA, -liveCA, -serverCert.
END
  exit $1
}

########## Process command-line ###############################################

ME=`my_basename $0`

RENEW_CA_PEMS=false
RENEW_DEFAULT_CA_PEMS=false
RENEW_SERVER_PEM=false

while true
do
  opt=`echo "$1" | tr '[A-Z]' '[a-z]'`  # make options case-insensitive
  case $opt in
  -help)
      usage 0
      ;;
  -defaultca)
      RENEW_DEFAULT_CA_PEMS=true
      ;;
  -liveca)
      RENEW_CA_PEMS=true
      ;;
  -opensslconf)
      [ "x$2" = "x" ] && usage 1
      OPENSSL_CONF="$2"
      shift
      ;;
  -servercert)
      RENEW_SERVER_PEM=true
      ;;
  -splunkhome)
      [ "x$2" = "x" ] && usage 1
      SPLUNK_HOME="$2"
      shift
      ;;
  -*)
      usage 1
      ;;
  *)
      break
      ;;
  esac
  shift
done
[ $# -eq 0 ] || usage 2

( $RENEW_CA_PEMS || $RENEW_DEFAULT_CA_PEMS || $RENEW_SERVER_PEM ) || usage 3

[ "x$OPENSSL_CONF" = "x" ] &&
  error "\$$OPENSSL_CONF not set; set or use -opensslConf" 4

[ -e "$OPENSSL_CONF" ] ||
  error "\$OPENSSL_CONF ($OPENSSL_CONF) does not exist" 5

[ -f "$OPENSSL_CONF" ] ||
  error "\$OPENSSL_CONF ($OPENSSL_CONF) is not a plain file" 6

[ "x$SPLUNK_HOME" = "x" ] &&
  error "\$SPLUNK_HOME not set; set or use -H" 7

[ -e "$SPLUNK_HOME" ] ||
  error "\$SPLUNK_HOME ($SPLUNK_HOME) does not exist" 8

[ -d "$SPLUNK_HOME" ] ||
  error "\$SPLUNK_HOME ($SPLUNK_HOME) is not a directory" 9

########## Check for required Unix commands ###################################

trap "x=$?; rm -f /tmp/_$$_*; exit $x" 0 1 2 15 # EXIT HUP INT TERM

PATH=$SPLUNK_HOME/bin:$PATH
export OPENSSL_CONF

openssl version >/dev/null 2>/dev/null ||
  error "openssl not found in \$PATH ($PATH)" 7

# It's very unlikely that 'sed' won't support -n; but we're being paranoid.
TMP_SED=/tmp/_$$_sed
sed -n 's/FOO/BAR/' $0 > "$TMP_SED" 2>/dev/null ||
  error "incompatible 'sed' version; must support -n option" 8
[ -s "$TMP_SED" ] &&
  error "incompatible 'sed' version; -n option doesn't work as expected" 9

########## Other variables ####################################################

AUTH_DIR="$SPLUNK_HOME/etc/auth"
CA_PEM="$AUTH_DIR/ca.pem"
CA_PEM_DEFAULT="$CA_PEM.default"
CACERT_PEM="$AUTH_DIR/cacert.pem"
CACERT_PEM_DEFAULT="$CACERT_PEM.default"
SERVER_PEM="$AUTH_DIR/server.pem"

CA_SUBJECT="/C=$CA_COUNTRY/ST=$CA_STATE/L=$CA_CITY/O=$CA_ORGANIZATION/CN=$CA_COMMON_NAME/emailAddress=$CA_E_MAIL/"
CERT_SUBJECT="/CN=$COMMON_NAME/O=SplunkUser/"

TMP_CERT_PEM=/tmp/_$$_cert.pem
TMP_CSR_PEM=/tmp/_$$_csr.pem
TMP_KEY_PEM=/tmp/_$$_key.pem

########## Renew default CA certificates ######################################

if $RENEW_DEFAULT_CA_PEMS
then
  assert_Splunk_cert_is_old "$CA_PEM_DEFAULT"
  assert_Splunk_cert_is_old "$CACERT_PEM_DEFAULT"

  extract_x509_private_key "$CA_PEM_DEFAULT" "$TMP_KEY_PEM"

  ##
  # Create certificate signing request (CSR).
  ##
  openssl req \
    -new -key "$TMP_KEY_PEM" \
    -passin "pass:$PASSWORD" \
    -sha1 \
    -subj "$CA_SUBJECT" \
    -out "$TMP_CSR_PEM"

  ##
  # Self-sign the renewed certificate with the root CA and CSR.
  ##
  backup "$CACERT_PEM_DEFAULT"
  openssl x509 \
    -req -in "$TMP_CSR_PEM" \
    -passin "pass:$PASSWORD" \
    -days $DAYS \
    -sha1 \
    -extensions v3_ca \
    -signkey "$TMP_KEY_PEM" \
    -out "$CACERT_PEM_DEFAULT"

  ##
  # Recreate the new ca.pem.default.
  ##
  backup "$CA_PEM_DEFAULT"
  cat "$CACERT_PEM_DEFAULT" "$TMP_KEY_PEM" > "$CA_PEM_DEFAULT"

  chmod 600 "$CA_PEM_DEFAULT"
  info "renewed default CA certificates"
fi

########## Renew live CA certificates #########################################

if $RENEW_CA_PEMS
then
  assert_Splunk_cert_is_new "$CA_PEM_DEFAULT"
  assert_Splunk_cert_is_new "$CACERT_PEM_DEFAULT"

  assert_Splunk_cert_is_old "$CA_PEM"
  assert_Splunk_cert_is_old "$CACERT_PEM"

  backup "$CA_PEM"
  backup "$CACERT_PEM"
  ( cp "$CA_PEM_DEFAULT" "$CA_PEM" &&
    cp "$CACERT_PEM_DEFAULT" "$CACERT_PEM" ) ||
      { restore "$CA_PEM"; restore "$CACERT_PEM"; exit 31; }

  chmod 600 "$CA_PEM" "$CACERT_PEM"
  info "renewed live CA certificates"
fi

########## Renew server.pem ###################################################

if $RENEW_SERVER_PEM
then
  assert_Splunk_cert_is_new "$CACERT_PEM"
  assert_is_Splunk_cert "$SERVER_PEM" >/dev/null

  extract_x509_first_cert "$SERVER_PEM" "$TMP_CERT_PEM"
  extract_x509_private_key "$SERVER_PEM" "$TMP_KEY_PEM"

  ##
  # Create a new server.pem with the original cert and private key, but the new
  # cacert.
  ##
  backup "$SERVER_PEM"
  cat "$TMP_CERT_PEM" "$TMP_KEY_PEM" "$CACERT_PEM" > "$SERVER_PEM" ||
    { restore "$SERVER_PEM"; exit 41; }

  chmod 600 "$SERVER_PEM"
  info "renewed server certificate"
fi

###############################################################################
# vim:set et sw=2 ts=2:
